@@ -48,6 +48,14 @@ sub new
    }, $class;
 }
 
+sub new_from_description
+{
+   my $class = shift;
+   my %args = @_;
+   # Whitelist known-OK constructor args
+   $class->new( map { $_ => $args{$_} } qw( serial baud ) );
+}
+
 =head1 METHODS
 
 This module provides no new methods beyond the basic API documented in
@@ -70,6 +78,21 @@ sub make_protocol_SPI
    });
 }
 
+sub make_protocol_I2C
+{
+    my $self = shift;
+
+    $self->{bp}->enter_mode( "I2C" )->then( sub {
+        my ( $mode ) = @_;
+        $self->{mode} = $mode;
+
+        $mode->configure( open_drain => 1 )
+            ->then_done(
+                Device::Chip::Adapter::BusPirate::_I2C->new( $mode )
+            );
+    });
+}
+
 sub shutdown
 {
    my $self = shift;
@@ -78,9 +101,7 @@ sub shutdown
 }
 
 package
-   Device::Chip::Adapter::BusPirate::_SPI;
-
-use Carp;
+   Device::Chip::Adapter::BusPirate::_base;
 
 use List::Util qw( first );
 
@@ -92,7 +113,27 @@ sub new
    bless { mode => $mode }, $class;
 }
 
-my @SPEEDS = (qw( 30k 125k 250k 1M 2M 2.6M 4M 8M ));
+sub _find_speed
+{
+   shift;
+   my ( $max_bitrate, @speeds ) = @_;
+
+    return first {
+        my $rate = $_;
+        $rate =~ s/k$/000/;
+        $rate =~ s/M$/000000/;
+
+        $rate <= $max_bitrate
+    } @speeds;
+}
+
+package
+   Device::Chip::Adapter::BusPirate::_SPI;
+use base qw( Device::Chip::Adapter::BusPirate::_base );
+
+use Carp;
+
+my @SPI_SPEEDS = (qw( 8M 4M 2.6M 2M 1M 250k 125k 30k ));
 
 sub configure
 {
@@ -105,20 +146,11 @@ sub configure
     croak "Unrecognised configuration options: " . join( ", ", keys %args )
         if %args;
 
-    # Translate 'max_bitrate' into the highest 'speed' setting the Bus Pirate
-    # can set
-    my $speed;
-    $speed = first {
-        my $rate = $_;
-        $rate =~ s/k$/000/;
-        $rate =~ s/M$/000000/;
-
-        $rate <= $max_bitrate
-    } @SPEEDS if defined $max_bitrate;
-
     $self->{mode}->configure(
-        ( defined $mode  ? ( mode  => $mode  ) : () ),
-        ( defined $speed ? ( speed => $speed ) : () ),
+        ( defined $mode ?
+           ( mode  => $mode ) : () ),
+        ( defined $max_bitrate ?
+           ( speed => $self->_find_speed( $max_bitrate, @SPI_SPEEDS ) ) : () ),
     );
 }
 
@@ -146,6 +178,62 @@ sub write
       ->then_done();
 }
 
+package
+    Device::Chip::Adapter::BusPirate::_I2C;
+
+use Carp;
+
+sub new
+{
+    my $class = shift;
+    my ( $mode ) = @_;
+
+    bless { mode => $mode }, $class;
+}
+
+my @I2C_SPEEDS = (qw( 400k 100k 50k 5k ));
+
+# TODO - addr ought to be a mount option somehow
+sub configure
+{
+    my $self = shift;
+    my %args = @_;
+
+    my $addr        = delete $args{addr};
+    my $max_bitrate = delete $args{max_bitrate};
+
+    croak "Unrecognised configuration options: " . join( ", ", keys %args )
+        if %args;
+
+    $self->{addr} = $addr if defined $addr;
+
+    my @f;
+
+    push @f, $self->{mode}->configure(
+       speed => $self->_find_speed( $max_bitrate, @I2C_SPEEDS )
+    ) if defined $max_bitrate;
+
+    Future->needs_all( @f );
+}
+
+sub power
+{
+   my $self = shift;
+   $self->{mode}->power( @_ );
+}
+
+sub write
+{
+    my $self = shift;
+    $self->{mode}->send( $self->{addr}, @_ );
+}
+
+sub write_then_read
+{
+    my $self = shift;
+    $self->{mode}->send_then_recv( $self->{addr}, @_ );
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
